<?php
/*
 * This action hook registers our PHP class as a WooCommerce payment gateway
 */
add_filter( 'woocommerce_payment_gateways', 'primepayments_add_gateway_class' );
function primepayments_add_gateway_class( $gateways ) {
	$gateways[] = 'WC_Primepayments_Gateway'; // your class name is here
	return $gateways;
}

/*
 * The class itself, please note that it is inside plugins_loaded action hook
 */
add_action( 'plugins_loaded', 'primepayments_init_gateway_class' );
function primepayments_init_gateway_class() {

	class WC_Primepayments_Gateway extends WC_Payment_Gateway {
		private $endpoint_url = 'https://pay.primepayments.io/API/v2/';
		private $secrey_key_1;
		private $secrey_key_2;

		/**
		 * Class constructor, more about it in Step 3
		 */
		public function __construct() {

			$this->id = 'primepayments'; // payment gateway plugin ID
			$this->icon = ''; // URL of the icon that will be displayed on checkout page near your gateway name
			$this->has_fields = true; // in case you need a custom credit card form
			$this->method_title = 'PrimePayments';
			$this->method_description = __('Take payments in person via PrimePayments', 'primepayments'); // will be displayed on the options page

			// gateways can support subscriptions, refunds, saved payment methods,
			// but in this tutorial we begin with simple payments
			$this->supports = array(
				'products'
			);

			// Method with all the options fields
			$this->init_form_fields();

			// Load the settings.
			$this->init_settings();
			$this->title 		= $this->get_option( 'title' );
			$this->description 	= $this->get_option( 'description' );
			$this->enabled 		= $this->get_option( 'enabled' );
			//$this->testmode 	= 'yes' === $this->get_option( 'testmode' );
			$this->project_id	= $this->get_option('project_id');
			$this->secret_key_1 = $this->get_option( 'secret_key_1' );
			$this->secret_key_2 = $this->get_option( 'secret_key_2' );

			// This action hook saves the settings
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

			// We need custom JavaScript to obtain a token
			add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );

			// You can also register a webhook here
			add_action( 'woocommerce_api_primepayments', array( $this, 'webhook' ) );

		}

		/**
		 * Plugin options, we deal with it in Step 3 too
		 */
		public function init_form_fields(){
			$this->form_fields = array(
				'enabled' => array(
					'title'       => __('Enable/Disable', 'primepayments'),
					'label'       => __('Enable PrimePayments Gateway', 'primepayments'),
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'title' => array(
					'title'       => __('Title', 'primepayments'),
					'type'        => 'text',
					'description' => __('This controls the title which the user sees during checkout.', 'primepayments'),
					'default'     => __('Credit Card', 'primepayments'),
					'desc_tip'    => true,
				),
				'description' => array(
					'title'       => __('Description', 'primepayments'),
					'type'        => 'textarea',
					'description' => __('This controls the description which the user sees during checkout.', 'primepayments'),
					'default'     => __('Pay with your credit card via our payment gateway.', 'primepayments'),
				),

				/*'testmode' => array(
					'title'       => __('Test mode', 'primepayments'),
					'label'       => __('Enable Test Mode', 'primepayments'),
					'type'        => 'checkbox',
					'description' => __('Place the payment gateway in test mode using test API keys.', 'primepayments'),
					'default'     => 'yes',
					'desc_tip'    => true,
				),*/

				'project_id' => array(
					'title'       => __('Project ID', 'primepayments'),
					'label'       => __('Project ID in primepayments dashboard', 'primepayments'),
					'type'        => 'text',
				),


				'secret_key_1' => array(
					'title'       => __('Secret Key 1', 'primepayments'),
					'type'        => 'text'
				),
				'secret_key_2' => array(
					'title'       => __('Secret Key 2', 'primepayments'),
					'type'        => 'text'
				),

				'webhook'	=> array(
					'title'			=> __('Endpoint URLs', 'primepayments'),
					'type'        => 'hidden',
					'description' =>
						'<h4>' . __('Set this URLs as `handler-script URLs` in your primepayments dashboard', 'primepayments') . '</h4>' .
						'<table>'.
						'<tr><th>' . __('Handler script URL: ', 'primepayments') . '</th><td>' .  get_site_url() . '/wc-api/primepayments/</td></tr>' .
						//'<tr><th>' .__('Redirect URL after successful payment: ') . '</th><td>' . get_site_url() . '/wc-api/primepayments?ppstatus=success</td></tr>'.
						//'<tr><th>' .__('Redirect URL after payment cancellation: ') . '</th><td>' . get_site_url() . '/wc-api/primepayments?ppstatus=cancel</td></tr>'.
						'</table>'

				)

			);
		}

		/**
		 * You will need it if you want your custom credit card form, Step 4 is about it
		 */
		public function payment_fields() {

			// ok, let's display some description before the payment form
			if ( $this->description ) {
				// you can instructions for test mode, I mean test card numbers etc.
				if ( $this->testmode ) {
					$this->description .= ' TEST MODE ENABLED. In test mode, you can use the card numbers listed in <a href="#">documentation</a>.';
					$this->description  = trim( $this->description );
				}
				// display the description with <p> tags etc.
				echo wpautop( wp_kses_post( $this->description ) );
			}

		}

		/*
		 * Custom CSS and JS, in most cases required only when you decided to go with a custom credit card form
		 */
		public function payment_scripts() {

			// we need JavaScript to process a token only on cart/checkout pages, right?
			if ( ! is_cart() && ! is_checkout() && ! isset( $_GET['pay_for_order'] ) ) {
				return;
			}

			// if our payment gateway is disabled, we do not have to enqueue JS too
			if ( 'no' === $this->enabled ) {
				return;
			}

			// no reason to enqueue JavaScript if API keys are not set
			if ( empty( $this->private_key ) || empty( $this->publishable_key ) ) {
				return;
			}

			// do not work with card detailes without SSL unless your website is in a test mode
			if ( ! $this->testmode && ! is_ssl() ) {
				return;
			}

		}

		/*
 		 * Fields validation, more in Step 5
		 */
		public function validate_fields() {
			return true;
		}

		/*
		 * We're processing the payments here, everything about it is in Step 5
		 */
		public function process_payment( $order_id ) {

			global $woocommerce;

			// we need it to get any order detailes
			$order = wc_get_order( $order_id );

			$action_type 		= 'initPayment';
			$payway 			= 0;
			$allowed_currencies = array('USD', 'RUB', 'UAH', 'EUR');
			$current_currency	= $order->get_currency();
			$lang 				= strtoupper( explode('-', get_bloginfo("language") )[0] );

			if( ! in_array($current_currency, $allowed_currencies) ){
				wc_add_notice(  __('This payment gateway doesn\'t support this currency, pleace select another method', 'primepayments'), 'error' );
				return;
			}

			/*
			  * Array with parameters for API interaction
			 */
			$args = array(
				'action' 			=> $action_type,
				'project'			=> $this->project_id,
				'sum'				=> $order->get_total(),
				'currency'			=> $current_currency,
				'email'				=> $order->get_billing_email(),
				'payWay'			=> $payway,
				'innerID'			=> $order->get_id(),
				//'orderID'			=> $order->get_id(),
				'returnLink'		=> 1,
				'needFailNotice'	=> 1,
				'lang'				=> $lang,
				'sign'				=> md5( $this->secret_key_1 . $action_type . $this->project_id . $order->get_total() . $order->get_currency() . $order->get_id() . $order->get_billing_email() . $payway),
			);

			/*
			 * Your API interaction could be built with wp_remote_post()
			  */
			$response = wp_remote_post( $this->endpoint_url, array(
				'method'	=> 'POST',
				'body'		=> $args
			) );


			if( !is_wp_error( $response ) ) {

				$body = json_decode( $response['body'], true );

				// it could be different depending on your payment processor
				if ( $body['status'] === 'OK' ) {

					$woocommerce->cart->empty_cart();

					// Redirect to the checkout page
					return array(
						'result' => 'success',
						'redirect' => $body['result']
					);

				} else {
					wc_add_notice(  __('Please try again.', 'primepayments'), 'error' );
					return;
				}

			} else {
				wc_add_notice(  __('Connection error.', 'primepayments'), 'error' );
				return;
			}

		}

		/*
		 * In case you need a webhook, like PayPal IPN etc
		 */
		public function webhook() {
			//=========================================================
			// Handle PAYED callback webhook from PrimePayments
			//=========================================================
			if( $_REQUEST['action'] === 'order_payed' ){
				// Validate sign
				$sign = md5( $this->secret_key_2 . $_REQUEST['orderID'] . $_REQUEST['payWay'] . $_REQUEST['innerID'] . $_REQUEST['sum'] . $_REQUEST['webmaster_profit'] );

				//TODO: Handle unsigned requests
				if( $sign !== $_REQUEST['sign'] ){
				    _e('Incorrect signature! Check the secret words in the settings.', 'primepayments');
				    exit;
				};

				$order_id = (int) $_REQUEST['innerID'];
				$order = wc_get_order( $order_id );
				$order->payment_complete();

				wc_reduce_stock_levels( $order );

				//На наш запрос сайт должен ответить "OK" (англ. символы, без кавычек)
				echo 'OK';
			}

			//=========================================================
			// !For future updates. Handle CANCELED callback webhook from PrimePayments
			//=========================================================
			else if( $_REQUEST['action'] === 'order_cancel' ){
				$sign = md5( $this->secret_key_2 . $_REQUEST['orderID'] . $_REQUEST['innerID'] );

				if( $sign !== $_REQUEST['sign'] ){
				    _e('Incorrect signature! Check the secret words in the settings.', 'primepayments');
				    exit;
				};

				$order = wc_get_order( $_POST['innerID'] );
				$order->update_status( 'failed', __( 'Payment canceled', 'primepayments' ) );

				//На наш запрос сайт должен ответить "OK" (англ. символы, без кавычек)
				echo 'OK';
			}


			//=========================================================
			// !For future updates. User redirected after successful payout
			//=========================================================
			else if ( isset( $_REQUEST['ppstatus'] ) && $_REQUEST['ppstatus'] == 'success' ) {
				$order = wc_get_order( (int) $_REQUEST['innerID'] );
				wp_redirect( $this->get_return_url( $order ) );
			}

			//=========================================================
			// User redirected after canceled payout
			//=========================================================
			else if ( isset( $_REQUEST['ppstatus'] ) && $_REQUEST['ppstatus'] == 'cancel' ) {
				$order = wc_get_order( $_REQUEST['innerID'] );
				$order->update_status( 'failed', __( 'Payment canceled', 'primepayments' ) );

				wp_redirect( $this->get_return_url( $order ) );
			}
			exit;
		}
	}
}
